@@ -1,13 +1,12 @@
 package JE::_FieldHash;
 
-our $VERSION = '0.016';
+our $VERSION = '0.017';
 
 
 use strict;
 use warnings;
 
 BEGIN {
-	local $@; # ~~~ Is this necessary in a BEGIN block?
 	eval { require Hash::Util::FieldHash;
 	       import  Hash::Util::FieldHash 'fieldhash'; };
 	if ($@) {
@@ -37,7 +36,7 @@ JE::_FieldHash - This module is solely for JE's private use.
 =head1 DESCRIPTION
 
 This is a thin wrapper around Hash::Util::FieldHash, or Tie::RefHash::Weak
-if the former is not available. B<It is subject to change or vanish without
+if the former is not available. B<It is liable to change or vanish without
 notice.>
 
 =head1 SEE ALSO
