@@ -1,7 +1,8 @@
 package JavaScript::Engine;
 
-our $VERSION = '0.016';
+our $VERSION = '0.017';
 
+use JE;
 
 =head1 NAME
 
@@ -12,7 +13,12 @@ JavaScript::Engine - Pure-Perl ECMAScript (JavaScript) engine
 This is just a pointer to L<JE>, a pure-Perl JS engine.
 
 "JE" stands for 
-JavaScript::Engine (following the example of PPI). I included this module
+JavaScript::Engine. I named it this, following the example of PPI, to avoid
+long class names like JavaScript::Engine::Object::Error::ReferenceError,
+which I thought would be a little ridiculous, and also time-consuming to
+type.
+
+I included this module
 in the JE distribution so that
 one might be able to find JE when typing C<i /javascript/> in the CPAN
 shell or when looking under modules/by-module/JavaScript/ on the CPAN.
